#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include "simpletools.h"
#include "config.h"
#include "debug.h"
#include "gcode_struct.h"
#include "adafruit_ads1015.h"
#include "sergcode.h"
#include "NewParser.h"
#include "x_axis_driver.h"
#include "y_axis_driver.h"
#include "z_axis_driver.h"
#include "e_axis_driver.h"

#define ALL_AXES_MAX_SPEED (CLKFREQ / 5714) // 14000 Rounded
#define ALL_AXES_PULSE_WIDTH (CLKFREQ / 1000000) // 80

static char fileline[100];
volatile bool app_move_now;

void calc_movement_timing()
{
	double total_high_pulse_time;
	double total_offset_time;
	double temp_offset_time;
	double total_time;

	if(current_gcode.x_total_steps > current_gcode.y_total_steps &&
		current_gcode.x_total_steps > current_gcode.z_total_steps &&
		current_gcode.x_total_steps > current_gcode.e_total_steps)
	{
		total_high_pulse_time = current_gcode.x_total_steps * ALL_AXES_PULSE_WIDTH;
		total_offset_time = current_gcode.x_total_steps * ALL_AXES_MAX_SPEED;
		total_time = total_high_pulse_time + total_offset_time;

		current_gcode.x_start_stop_speed = ALL_AXES_MAX_SPEED;

		if(current_gcode.y_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.y_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.y_total_steps);

			current_gcode.y_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.y_start_stop_speed = 0;
		}

		if(current_gcode.z_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.z_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.z_total_steps);

			current_gcode.z_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.z_start_stop_speed = 0;
		}

		if(current_gcode.e_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.e_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.e_total_steps);

			current_gcode.e_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.e_start_stop_speed = 0;
		}
	}
	else if(current_gcode.y_total_steps > current_gcode.x_total_steps &&
		current_gcode.y_total_steps > current_gcode.z_total_steps &&
		current_gcode.y_total_steps > current_gcode.e_total_steps)
	{
		total_high_pulse_time = current_gcode.y_total_steps * ALL_AXES_PULSE_WIDTH;
		total_offset_time = current_gcode.y_total_steps * ALL_AXES_MAX_SPEED;
		total_time = total_high_pulse_time + total_offset_time;

		current_gcode.y_start_stop_speed = ALL_AXES_MAX_SPEED;

		if(current_gcode.x_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.x_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.x_total_steps);

			current_gcode.x_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.x_start_stop_speed = 0;
		}

		if(current_gcode.z_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.z_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.z_total_steps);

			current_gcode.z_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.z_start_stop_speed = 0;
		}

		if(current_gcode.e_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.e_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.e_total_steps);

			current_gcode.e_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.e_start_stop_speed = 0;
		}
	}
	else if(current_gcode.z_total_steps > current_gcode.y_total_steps &&
		current_gcode.z_total_steps > current_gcode.x_total_steps &&
		current_gcode.z_total_steps > current_gcode.e_total_steps)
	{
		total_high_pulse_time = current_gcode.z_total_steps * ALL_AXES_PULSE_WIDTH;
		total_offset_time = current_gcode.z_total_steps * ALL_AXES_MAX_SPEED;
		total_time = total_high_pulse_time + total_offset_time;

		current_gcode.z_start_stop_speed = ALL_AXES_MAX_SPEED;

		if(current_gcode.x_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.x_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.x_total_steps);

			current_gcode.x_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.x_start_stop_speed = 0;
		}

		if(current_gcode.y_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.y_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.y_total_steps);

			current_gcode.y_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.y_start_stop_speed = 0;
		}

		if(current_gcode.e_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.e_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.e_total_steps);

			current_gcode.e_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.e_start_stop_speed = 0;
		}
	}
	else if(current_gcode.e_total_steps > current_gcode.x_total_steps &&
		current_gcode.e_total_steps > current_gcode.y_total_steps &&
		current_gcode.e_total_steps > current_gcode.z_total_steps)
	{
		total_high_pulse_time = current_gcode.e_total_steps * ALL_AXES_PULSE_WIDTH;
		total_offset_time = current_gcode.e_total_steps * ALL_AXES_MAX_SPEED;
		total_time = total_high_pulse_time + total_offset_time;

		current_gcode.e_start_stop_speed = ALL_AXES_MAX_SPEED;

		if(current_gcode.x_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.x_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.x_total_steps);

			current_gcode.x_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.x_start_stop_speed = 0;
		}

		if(current_gcode.y_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.y_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.y_total_steps);

			current_gcode.y_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.y_start_stop_speed = 0;
		}

		if(current_gcode.z_total_steps > 0)
		{
			temp_offset_time = total_time - ((current_gcode.z_total_steps *
				ALL_AXES_PULSE_WIDTH) / current_gcode.z_total_steps);

			current_gcode.z_start_stop_speed = (uint32_t)floor(temp_offset_time + 0.5);
		}
		else
		{
			current_gcode.z_start_stop_speed = 0;
		}
	}
}

int calc_steps_required(double travel, int32_t steps_per_mm)
{
	double steps;

	steps = travel * (double)steps_per_mm;

	return (uint32_t)floor(steps + 0.5);
}

void perform_gcode_struct_calcs()
{
	if(current_gcode.x_num != 0 && current_gcode.x_num > previous_gcode.x_num)
	{
		current_gcode.x_travel = current_gcode.x_num - previous_gcode.x_num;
		current_gcode.x_dir = X_INC_DIR;
		current_gcode.x_total_steps = 
			calc_steps_required(current_gcode.x_travel, X_STEPS_PER_MM);
	}     
	else if(current_gcode.x_num != 0 && current_gcode.x_num < previous_gcode.x_num)
	{
		current_gcode.x_travel = previous_gcode.x_num - current_gcode.x_num;
		current_gcode.x_dir = X_DEC_DIR;
		current_gcode.x_total_steps = 
			calc_steps_required(current_gcode.x_travel, X_STEPS_PER_MM);
	} 
	else
	{
		current_gcode.x_travel = 0;
		current_gcode.x_dir = previous_gcode.x_dir;
		current_gcode.x_total_steps = 0;
	}

	if(current_gcode.y_num != 0 && current_gcode.y_num > previous_gcode.y_num)
	{
		current_gcode.y_travel = current_gcode.y_num - previous_gcode.y_num;
		current_gcode.y_dir = Y_INC_DIR;
		current_gcode.y_total_steps = 
			calc_steps_required(current_gcode.y_travel, Y_STEPS_PER_MM);
	}     
	else if(current_gcode.y_num != 0 && current_gcode.y_num < previous_gcode.y_num)
	{
		current_gcode.y_travel = previous_gcode.y_num - current_gcode.y_num;
		current_gcode.y_dir = Y_DEC_DIR;
		current_gcode.y_total_steps = 
			calc_steps_required(current_gcode.y_travel, Y_STEPS_PER_MM);
	} 
	else
	{
		current_gcode.y_travel = 0;
		current_gcode.y_dir = previous_gcode.y_dir;
		current_gcode.y_total_steps = 0;
	}

	if(current_gcode.z_num != 0 && current_gcode.z_num > previous_gcode.z_num)
	{
		current_gcode.z_travel = current_gcode.z_num - previous_gcode.z_num;
		current_gcode.z_dir = Z_INC_DIR;
		current_gcode.z_total_steps = 
			calc_steps_required(current_gcode.z_travel, Z_STEPS_PER_MM);
	}     
	else if(current_gcode.z_num != 0 && current_gcode.z_num < previous_gcode.z_num)
	{
		current_gcode.z_travel = previous_gcode.z_num - current_gcode.z_num;
		current_gcode.z_dir = Z_DEC_DIR;
		current_gcode.z_total_steps = 
			calc_steps_required(current_gcode.z_travel, Z_STEPS_PER_MM);
	} 
	else
	{
		current_gcode.z_travel = 0;
		current_gcode.z_dir = previous_gcode.z_dir;
		current_gcode.z_total_steps = 0;
	}

	if(current_gcode.e_num != 0 && current_gcode.e_num > previous_gcode.e_num)
	{
		current_gcode.e_travel = current_gcode.e_num - previous_gcode.e_num;
		current_gcode.e_dir = E_INC_DIR;
		current_gcode.e_total_steps = 
			calc_steps_required(current_gcode.e_travel, E_STEPS_PER_MM);
	}     
	else if(current_gcode.e_num != 0 && current_gcode.e_num < previous_gcode.e_num)
	{
		current_gcode.e_travel = previous_gcode.e_num - current_gcode.e_num;
		current_gcode.e_dir = E_DEC_DIR;
		current_gcode.e_total_steps = 
			calc_steps_required(current_gcode.e_travel, E_STEPS_PER_MM);
	} 
	else
	{
		current_gcode.e_travel = 0;
		current_gcode.e_dir = previous_gcode.e_dir;
		current_gcode.e_total_steps = 0;
	}

	// Calculate step timing when necessary
	if((current_gcode.valid[0] == '1' && current_gcode.g_num == 1) ||
		(current_gcode.valid[0] == '1' && current_gcode.g_num == 0))
	{
		calc_movement_timing();
	}
}

void ini_gcode_struct()
{
	uint8_t gcode_char;
	uint8_t gcode_struct_element;
	uint8_t fileline_length;
	uint8_t value_counter;

	uint32_t counter;

	// Clear any lingering data
	memset(current_gcode.gcode_fileline, 0, strlen(current_gcode.gcode_fileline));
	memset(current_gcode.valid, 0, strlen(current_gcode.valid));
	memset(current_gcode.g_char, 0, strlen(current_gcode.g_char));
	memset(current_gcode.m_char, 0, strlen(current_gcode.m_char));
	memset(current_gcode.t_char, 0, strlen(current_gcode.t_char));
	memset(current_gcode.s_char, 0, strlen(current_gcode.s_char));
	memset(current_gcode.p_char, 0, strlen(current_gcode.p_char));
	memset(current_gcode.x_char, 0, strlen(current_gcode.x_char));
	memset(current_gcode.y_char, 0, strlen(current_gcode.y_char));
	memset(current_gcode.z_char, 0, strlen(current_gcode.z_char));
	memset(current_gcode.e_char, 0, strlen(current_gcode.e_char));
	memset(current_gcode.f_char, 0, strlen(current_gcode.f_char));

	memcpy(&current_gcode.gcode_fileline, &fileline, sizeof(fileline));

	// Get the length of the fileline that is being parsed
	fileline_length = (uint8_t)strlen(fileline);

	// Fill current_gcode.valid with literal zeros
	for(counter = 0; counter < GCODE_ELEMENTS; counter++)
	{
		current_gcode.valid[counter] = '0';
	}

	value_counter = 0;

	//Parse file line into seperate field values.  Various fields will remain empty
	for(counter = 0; counter < fileline_length; counter++)
	{
		gcode_char = fileline[counter];

		// Exclude these characters from becoming part of are field entries
		if(gcode_char == 13 /*Carriage Return*/ || gcode_char == 10 /*Line Feed*/)
		{
			// Break free from this loop
			break;
		}

		else if(gcode_char == ' ')
		{
			value_counter = 0;

			// Skip the remaining iterations, because
			// we found a match, however, stay in the loop,
			// because we are not done yet
			continue;
		}

		else if(gcode_char == 'G')
		{
			// Set current field element
			gcode_struct_element = 0;
			current_gcode.valid[0] = '1';

			continue;
		}
		else if(gcode_char == 'M')
		{
			// Set current field element
			gcode_struct_element = 1;
			current_gcode.valid[1] = '1';

			continue;
		}
		else if(gcode_char == 'T')
		{
			// Set current field element
			gcode_struct_element = 2;
			current_gcode.valid[2] = '1';

			continue;
		}
		else if(gcode_char == 'S')
		{
			// Set current field element
			gcode_struct_element = 3;
			current_gcode.valid[3] = '1';

			continue;
		}
		else if(gcode_char == 'P')
		{
			// Set current field element
			gcode_struct_element = 4;
			current_gcode.valid[4] = '1';

			continue;
		}
		else if(gcode_char == 'X')
		{
			// Set current field element
			gcode_struct_element = 5;
			current_gcode.valid[5] = '1';

			continue;
		}
		else if(gcode_char == 'Y')
		{
			// Set current field element
			gcode_struct_element = 6;
			current_gcode.valid[6] = '1';

			continue;
		}
		else if(gcode_char == 'Z')
		{
			// Set current field element
			gcode_struct_element = 7;
			current_gcode.valid[7] = '1';

			continue;
		}		
		else if(gcode_char == 'E')
		{
			// Set current field element
			gcode_struct_element = 8;
			current_gcode.valid[8] = '1';

			continue;
		}
		else if(gcode_char == 'F')
		{
			// Set current field element
			gcode_struct_element = 9;
			current_gcode.valid[9] = '1';

			continue;
		}
		else
		{
			if(gcode_struct_element == 0)
			{
				// G Field element character
				current_gcode.g_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 1)
			{
				// M Field element character
				current_gcode.m_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 2)
			{
				// T Field element character
				current_gcode.t_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 3)
			{
				// S Field element character
				current_gcode.s_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 4)
			{
				// P Field element character
				current_gcode.p_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 5)
			{
				// X Field element character
				current_gcode.x_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 6)
			{
				// Y Field element character
				current_gcode.y_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 7)
			{
				// Z Field element character
				current_gcode.z_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 8)
			{
				// E Field element character
				current_gcode.e_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}
			else if(gcode_struct_element == 9)
			{
				// F Field element character
				current_gcode.f_char[value_counter] = gcode_char;
				value_counter++;

				continue;
			}			
		}
	}

	current_gcode.g_num = atoi(current_gcode.g_char);
	current_gcode.m_num = atoi(current_gcode.m_char);
	current_gcode.t_num = atoi(current_gcode.t_char);
	current_gcode.s_num = atoi(current_gcode.s_char);
	current_gcode.p_num = atoi(current_gcode.p_char);

	current_gcode.x_num = atof(current_gcode.x_char);
	current_gcode.y_num = atof(current_gcode.y_char);
	current_gcode.z_num = atof(current_gcode.z_char);
	current_gcode.e_num = atof(current_gcode.e_char);
	current_gcode.f_num = atof(current_gcode.f_char);

	perform_gcode_struct_calcs();

	////////////////////////////////////////////////////////////
	// The following assignments must be removed when the
	// calc_movement_timing() function has been properly defined
	current_gcode.x_running_steps = current_gcode.x_total_steps;
	current_gcode.y_running_steps = current_gcode.y_total_steps;
	current_gcode.z_running_steps = current_gcode.z_total_steps;
	current_gcode.e_running_steps = current_gcode.e_total_steps;
	////////////////////////////////////////////////////////////
}

int main(void)
{
	app_move_now = false;

	adc_init();
//	serial_init();

	/////////////////////////////////////////////////////////////
	// Test X, Y, Z, and E axes for syncronization with G1 commands.
	// This will most likely be moved to another Propeller.
	/////////////////////////////////////////////////////////////
	x_axis_init();
	y_axis_init();
	z_axis_init();
	e_axis_init();

	x_move_complete = true;
	y_move_complete = true;
	z_move_complete = true;
	e_move_complete = true;
	/////////////////////////////////////////////////////////////

	uint32_t line_counter = 0;

	// Set debug level
	debug_flags |= DEBUG_ERROR;

	// Initial previous_gcode GCODE_STRUCT members to zero
	previous_gcode.g_num = 0;
	previous_gcode.m_num = 0;
	previous_gcode.t_num = 0;
	previous_gcode.s_num = 0;
	previous_gcode.p_num = 0;
	previous_gcode.x_num = 0;
	previous_gcode.y_num = 0;
	previous_gcode.z_num = 0;
	previous_gcode.e_num = 0;
	previous_gcode.f_num = 0;

	previous_gcode.x_travel = 0;
	previous_gcode.y_travel = 0;
	previous_gcode.z_travel = 0;
	previous_gcode.e_travel = 0;

	previous_gcode.x_dir = 0; 
	previous_gcode.x_start_stop_speed = 0;
	previous_gcode.x_total_steps = 0;
	previous_gcode.x_ramp_up_steps = 0;
	previous_gcode.x_running_steps = 0;
	previous_gcode.x_ramp_down_steps = 0;
	previous_gcode.x_ramp_inc_dec = 0;

	previous_gcode.y_dir = 0;
	previous_gcode.y_start_stop_speed = 0;
	previous_gcode.y_total_steps = 0;
	previous_gcode.y_ramp_up_steps = 0;
	previous_gcode.y_running_steps = 0;
	previous_gcode.y_ramp_down_steps = 0;
	previous_gcode.y_ramp_inc_dec = 0;

	previous_gcode.z_dir = 0;
	previous_gcode.z_start_stop_speed = 0;
	previous_gcode.z_total_steps = 0;
	previous_gcode.z_ramp_up_steps = 0;
	previous_gcode.z_running_steps = 0;
	previous_gcode.z_ramp_down_steps = 0;
	previous_gcode.z_ramp_inc_dec = 0;

	previous_gcode.e_dir = 0;
	previous_gcode.e_start_stop_speed = 0;
	previous_gcode.e_total_steps = 0;
	previous_gcode.e_ramp_up_steps = 0;
	previous_gcode.e_running_steps = 0;
	previous_gcode.e_ramp_down_steps = 0;
	previous_gcode.e_ramp_inc_dec = 0;

	// Verify that SD card pins are defined
	#if defined DO && defined CLK && defined DI && defined SD_CS

	// Mount SD card
	sd_mount(DO, CLK, DI, SD_CS);

	FILE * pFile;

	// Open up a file on the SD card
	pFile = fopen("DEBUG.TXT","r");

	if(pFile == NULL)
	{
		if(DEBUG_ERROR && (debug_flags & DEBUG_ERROR))
		{
			print("ERROR - The file could not be opened!");
		}
	}
	else
	{
		// Recursively grab filelines
		while(fgets(fileline, 100, pFile))
		{
			// Count the filelines as we go along
			line_counter++;

			// Exclude lines that begin with certain characters
			// E.G. Remove newlines, carriage returns, and lines
			// that begin with a semi-colon.
			if(fileline[0] != 59 && fileline[0] != 13 && fileline[0] != 10)
			{
				// Fill the GCODE_STRUCT with the data
				// obtained from the fileline
				ini_gcode_struct();

				/////////////////////////////////////////////////////////////
				// Test X and Y axes for syncronization with G1 commands.
				// E.G. G1 X10 Y15 Z20 E25...G1 X25 Y30 Z35 E40...etc
				// This will most likely be moved to another Propeller
				/////////////////////////////////////////////////////////////
				app_move_now = true;

				while(x_move_complete == false && y_move_complete == false && 
					z_move_complete == false && e_move_complete == false){}

				x_move_complete = false;
				y_move_complete = false;
				z_move_complete = false;
				e_move_complete = false;
				/////////////////////////////////////////////////////////////

				// Trim off newline character from the fileline
				// returned by fgets
				fileline[strcspn(fileline, "\n")] = 0;

    				print("Fileline: %s\n", fileline);

    				print("valid GCODE Fields: %s\n", current_gcode.valid);

				// Check the various GCODE_STRUCT field lengths
				// to see if they contain data, and if so, then
				// print the data
				if((uint8_t)strlen(current_gcode.g_char) != 0)
				{
					print("G: %s, ", current_gcode.g_char);
				}

				if((uint8_t)strlen(current_gcode.m_char) != 0)
				{
					print("M: %s, ", current_gcode.m_char);
				}

				if((uint8_t)strlen(current_gcode.t_char) != 0)
				{
					print("T: %s, ", current_gcode.t_char);
				}

				if((uint8_t)strlen(current_gcode.s_char) != 0)
				{
					print("S: %s, ", current_gcode.s_char);
				}

				if((uint8_t)strlen(current_gcode.p_char) != 0)
				{
					print("P: %s, ", current_gcode.p_char);
				}

				if((uint8_t)strlen(current_gcode.x_char) != 0)
				{
					print("X: %s, ", current_gcode.x_char);
				}

				if((uint8_t)strlen(current_gcode.y_char) != 0)
				{
					print("Y: %s, ", current_gcode.y_char);
				}

				if((uint8_t)strlen(current_gcode.z_char) != 0)
				{
					print("Z: %s, ", current_gcode.z_char);
				}

				if((uint8_t)strlen(current_gcode.e_char) != 0)
				{
					print("E: %s, ", current_gcode.e_char);
				}

				if((uint8_t)strlen(current_gcode.f_char) != 0)
				{
					print("F: %s, ", current_gcode.f_char);
				}

				print("\n");

				if((uint8_t)strlen(current_gcode.g_char) != 0)
				{
					print("G: %u, ", current_gcode.g_num);
				}

				if((uint8_t)strlen(current_gcode.m_char) != 0)
				{
					print("M: %u, ", current_gcode.m_num);
				}

				if((uint8_t)strlen(current_gcode.t_char) != 0)
				{
					print("T: %u, ", current_gcode.t_num);
				}

				if((uint8_t)strlen(current_gcode.s_char) != 0)
				{
					print("S: %u, ", current_gcode.s_num);
				}

				if((uint8_t)strlen(current_gcode.p_char) != 0)
				{
					print("P: %u, ", current_gcode.p_num);
				}

				if((uint8_t)strlen(current_gcode.x_char) != 0)
				{
					print("X: %5.2f, ", current_gcode.x_num);
				}

				if((uint8_t)strlen(current_gcode.y_char) != 0)
				{
					print("Y: %5.2f, ", current_gcode.y_num);
				}

				if((uint8_t)strlen(current_gcode.z_char) != 0)
				{
					print("Z: %5.2f, ", current_gcode.z_num);
				}

				if((uint8_t)strlen(current_gcode.e_char) != 0)
				{
					print("E: %5.2f, ", current_gcode.e_num);
				}

				if((uint8_t)strlen(current_gcode.f_char) != 0)
				{
					print("F: %5.2f, ", current_gcode.f_num);
				}

				print("\n");

				print("Clock Frequency: %u\n", CLKFREQ);

				print("Total Steps Required: X: %u, Y: %u, Z: %u, E: %u\n",
					current_gcode.x_total_steps, current_gcode.y_total_steps,
					current_gcode.z_total_steps, current_gcode.e_total_steps);

				print("Step Pulse Offset: X: %u, Y: %u, Z: %u, E: %u\n",
					current_gcode.x_start_stop_speed, current_gcode.y_start_stop_speed,
					current_gcode.z_start_stop_speed, current_gcode.e_start_stop_speed);

				print("Distance: X: %5.2f, Y: %5.2f, Z: %5.2f, E: %5.2f\n",
					current_gcode.x_travel, current_gcode.y_travel,
					current_gcode.z_travel, current_gcode.e_travel);

				print("Motor Rotation: X: %u, Y: %u, Z: %u, E: %u\n",
					current_gcode.x_dir, current_gcode.y_dir,
					current_gcode.z_dir, current_gcode.e_dir);

				print("Previous Values: G: %u, ", previous_gcode.g_num);
				print("M: %u, ", previous_gcode.m_num);
				print("T: %u, ", previous_gcode.t_num);
				print("S: %u, ", previous_gcode.s_num);
				print("P: %u, ", previous_gcode.p_num);
				print("X: %5.2f, ", previous_gcode.x_num);
				print("Y: %5.2f, ", previous_gcode.y_num);
				print("Z: %5.2f, ", previous_gcode.z_num);
				print("E: %5.2f, ", previous_gcode.e_num);
				print("F: %5.2f, ", previous_gcode.f_num);
				print("\nPrevious Total Steps Required: X: %u, Y: %u, Z: %u, E: %u",
					previous_gcode.x_total_steps, previous_gcode.y_total_steps,
					previous_gcode.z_total_steps, previous_gcode.e_total_steps);

				print("\n\n");
			}

			// Copy the current GCODE_STRUCT into the previous one
			memcpy(&previous_gcode, &current_gcode, sizeof(GCODE_STRUCT));

			// Clear any lingering data from the fileline variable
			memset(fileline, 0, strlen(fileline));
		}

		// Close up the file on the SD card
		fclose(pFile);

		// Print the number of filelines encountered
		print("%u total lines", line_counter);
	}
 
	#else
    
	if(DEBUG_ERROR && (debug_flags & DEBUG_ERROR))
	{
		print("ERROR - The proper inputs and outputs must be defined for SD card usage!");
	}
    
	#endif

	/////////////////////////////////////////////////////////////
	// Test X, Y, Z, and E axes for syncronization with G1 commands.
	// This will most likely be moved to another Propeller.
	/////////////////////////////////////////////////////////////
//	x_axis_stop();
//	y_axis_stop();
//	z_axis_stop();
//	e_axis_stop();
	/////////////////////////////////////////////////////////////    

	return 0;
}