#include "e_axis_driver.h"
#include "config.h"
#include "gcode_struct.h"
#include "NewParser.h"

/// cog variables
static int e_axis_cog = 0;
static int e_axis_stack[100];

volatile bool e_move_complete;

void e_axis_driver(void *par);

void e_axis_init()
{
	if(e_axis_cog == 0)
	{
		e_axis_cog = 1 + cogstart(&e_axis_driver, NULL,
			e_axis_stack, sizeof e_axis_stack);
	}
}

/// stop e_axis cog
void e_axis_stop()
{
	if(e_axis_cog > 0)
	{
		cogstop(e_axis_cog - 1);
		e_axis_cog = 0;
	}
}

void e_axis_driver(void *par)
{
	int32_t e_counter;

	// Set DIRA as an output.
	DIRA |= 1 << E_DIRECTION;

	do
	{
		if(app_move_now == true)
		{
			app_move_now = false;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = 4 << 26 | E_STEP;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << E_STEP;

			// Set the OUTA register to match the desired direction of rotation.
			if(current_gcode.e_dir == 0)
			{
				OUTA &= ~1 << E_DIRECTION;
			}
			else
			{
				OUTA |= 1 << E_DIRECTION;
			}

			// Get the current System Counter value.
			e_counter = CNT;

			// Ramp up the stepper motor a predetermined amount of steps
			while(current_gcode.e_ramp_up_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -E_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(e_counter += current_gcode.e_start_stop_speed -= current_gcode.e_ramp_inc_dec);

				// Decrement ramp_up_steps
				current_gcode.e_ramp_up_steps--;
			}

			// Run the stepper motor at the current speed for a predetermined amount of steps
			while(current_gcode.e_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -E_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(e_counter += current_gcode.e_start_stop_speed);

				// Decrement running_steps
				current_gcode.e_running_steps--;
			}

			// Ramp down the stepper motor a predetermined amount of steps
			while(current_gcode.e_ramp_down_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -E_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(e_counter += current_gcode.e_start_stop_speed += current_gcode.e_ramp_inc_dec);

				// Decrement ramp_down_steps
				current_gcode.e_ramp_down_steps--;
			}

			e_move_complete = true;
		}
	}

	while(e_axis_cog > 0);
}