#include "x_axis_driver.h"
#include "config.h"
#include "gcode_struct.h"
#include "NewParser.h"

/// cog variables
static int x_axis_cog = 0;
static int x_axis_stack[100];

volatile bool x_move_complete;

void x_axis_driver(void *par);

void x_axis_init()
{
	if(x_axis_cog == 0)
	{
		x_axis_cog = 1 + cogstart(&x_axis_driver, NULL,
			x_axis_stack, sizeof x_axis_stack);
	}
}

/// stop x_axis cog
void x_axis_stop()
{
	if(x_axis_cog > 0)
	{
		cogstop(x_axis_cog - 1);
		x_axis_cog = 0;
	}
}

void x_axis_driver(void *par)
{
	int32_t x_counter;

	// Set DIRA as an output.
	DIRA |= 1 << X_DIRECTION;

	do
	{
		if(app_move_now == true)
		{
			app_move_now = false;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = 4 << 26 | X_STEP;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << X_STEP;

			// Set the OUTA register to match the desired direction of rotation.
			if(current_gcode.x_dir == 0)
			{
				OUTA &= ~1 << X_DIRECTION;
			}
			else
			{
				OUTA |= 1 << X_DIRECTION;
			}

			// Get the current System Counter value.
			x_counter = CNT;

			// Ramp up the stepper motor a predetermined amount of steps
			while(current_gcode.x_ramp_up_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -X_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(x_counter += current_gcode.x_start_stop_speed -= current_gcode.x_ramp_inc_dec);

				// Decrement ramp_up_steps
				current_gcode.x_ramp_up_steps--;
			}

			// Run the stepper motor at the current speed for a predetermined amount of steps
			while(current_gcode.x_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -X_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(x_counter += current_gcode.x_start_stop_speed);

				// Decrement running_steps
				current_gcode.x_running_steps--;
			}

			// Ramp down the stepper motor a predetermined amount of steps
			while(current_gcode.x_ramp_down_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -X_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(x_counter += current_gcode.x_start_stop_speed += current_gcode.x_ramp_inc_dec);

				// Decrement ramp_down_steps
				current_gcode.x_ramp_down_steps--;
			}

			x_move_complete = true;
		}
	}

	while(x_axis_cog > 0);
}