#include "y_axis_driver.h"
#include "config.h"
#include "gcode_struct.h"
#include "NewParser.h"

/// cog variables
static int y_axis_cog = 0;
static int y_axis_stack[100];

volatile bool y_move_complete;

void y_axis_driver(void *par);

void y_axis_init()
{
	if(y_axis_cog == 0)
	{
		y_axis_cog = 1 + cogstart(&y_axis_driver, NULL,
			y_axis_stack, sizeof y_axis_stack);
	}
}

/// stop y_axis cog
void y_axis_stop()
{
	if(y_axis_cog > 0)
	{
		cogstop(y_axis_cog - 1);
		y_axis_cog = 0;
	}
}

void y_axis_driver(void *par)
{
	int32_t y_counter;

	// Set DIRA as an output.
	DIRA |= 1 << Y_DIRECTION;

	do
	{
		if(app_move_now == true)
		{
			app_move_now = false;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = 4 << 26 | Y_STEP;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << Y_STEP;

			// Set the OUTA register to match the desired direction of rotation.
			if(current_gcode.y_dir == 0)
			{
				OUTA &= ~1 << Y_DIRECTION;
			}
			else
			{
				OUTA |= 1 << Y_DIRECTION;
			}

			// Get the current System Counter value.
			y_counter = CNT;

			// Ramp up the stepper motor a predetermined amount of steps
			while(current_gcode.y_ramp_up_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Y_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(y_counter += current_gcode.y_start_stop_speed -= current_gcode.y_ramp_inc_dec);

				// Decrement ramp_up_steps
				current_gcode.y_ramp_up_steps--;
			}

			// Run the stepper motor at the current speed for a predetermined amount of steps
			while(current_gcode.y_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Y_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(y_counter += current_gcode.y_start_stop_speed);

				// Decrement running_steps
				current_gcode.y_running_steps--;
			}

			// Ramp down the stepper motor a predetermined amount of steps
			while(current_gcode.y_ramp_down_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Y_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(y_counter += current_gcode.y_start_stop_speed += current_gcode.y_ramp_inc_dec);

				// Decrement ramp_down_steps
				current_gcode.y_ramp_down_steps--;
			}

			y_move_complete = true;
		}
	}

	while(y_axis_cog > 0);
}