#include "z_axis_driver.h"
#include "config.h"
#include "gcode_struct.h"
#include "NewParser.h"

/// cog variables
static int z_axis_cog = 0;
static int z_axis_stack[100];

volatile bool z_move_complete;

void z_axis_driver(void *par);

void z_axis_init()
{
	if(z_axis_cog == 0)
	{
		z_axis_cog = 1 + cogstart(&z_axis_driver, NULL,
			z_axis_stack, sizeof z_axis_stack);
	}
}

/// stop z_axis cog
void z_axis_stop()
{
	if(z_axis_cog > 0)
	{
		cogstop(z_axis_cog - 1);
		z_axis_cog = 0;
	}
}

void z_axis_driver(void *par)
{
	int32_t z_counter;

	// Set DIRA as an output.
	DIRA |= 1 << Z_DIRECTION;

	do
	{
		if(app_move_now == true)
		{
			app_move_now = false;

			// Set up the CTRMODE of Counter A for NCO/PWM single-ended.
			CTRA = 4 << 26 | Z_STEP;

			// Set the value to be added to PHSA with every clock cycle.
			FRQA = 1;

			// Set DIRA as an output.
			DIRA |= 1 << Z_STEP;

			// Set the OUTA register to match the desired direction of rotation.
			if(current_gcode.z_dir == 0)
			{
				OUTA &= ~1 << Z_DIRECTION;
			}
			else
			{
				OUTA |= 1 << Z_DIRECTION;
			}

			// Get the current System Counter value.
			z_counter = CNT;

			// Ramp up the stepper motor a predetermined amount of steps
			while(current_gcode.z_ramp_up_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Z_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(z_counter += current_gcode.z_start_stop_speed -= current_gcode.z_ramp_inc_dec);

				// Decrement ramp_up_steps
				current_gcode.z_ramp_up_steps--;
			}

			// Run the stepper motor at the current speed for a predetermined amount of steps
			while(current_gcode.z_running_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Z_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(z_counter += current_gcode.z_start_stop_speed);

				// Decrement running_steps
				current_gcode.z_running_steps--;
			}

			// Ramp down the stepper motor a predetermined amount of steps
			while(current_gcode.z_ramp_down_steps != 0)
			{
				// Send out a high pulse on the step pin for the desired duration.
				PHSA = -Z_PULSE_WIDTH;

				// Wait for a specified period of time before sending another
				// high pulse to the step pin.
				waitcnt(z_counter += current_gcode.z_start_stop_speed += current_gcode.z_ramp_inc_dec);

				// Decrement ramp_down_steps
				current_gcode.z_ramp_down_steps--;
			}

			z_move_complete = true;
		}
	}

	while(z_axis_cog > 0);
}